/' Copyright (c) 2007 Scott Lembcke
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 '/
 
'' Number of frames that contact information should persist.
extern cp_contact_persistence alias "cp_contact_persistence" as cpTimestamp

'' User collision handler function types.
type cpCollisionBeginFunc as function cdecl( byval as cpArbiter ptr, byval as any ptr, byval as any ptr ) as cpFloat
type cpCollisionPreSolveFunc as function cdecl( byval as cpArbiter ptr, byval as any ptr, byval as any ptr ) as cpFloat
type cpCollisionPostSolveFunc as sub cdecl( byval as cpArbiter ptr, byval as any ptr, byval as any ptr )
type cpCollisionSeparateFunc as sub cdecl( byval as cpArbiter ptr, byval as any ptr, byval as any ptr )

'' Structure for holding collision pair function information.
'' Used internally.
type cpCollisionHandler
	as cpCollisionType			a
	as cpCollisionType			b
	as cpCollisionBeginFunc		begin
	as cpCollisionPreSolveFunc	preSolve
	as cpCollisionPostSolveFunc	postSolve
	as cpCollisionSeparateFunc	separate
	as any ptr					data
end type


#define CP_MAX_CONTACTS_PER_ARBITER 6
type cpContactBufferHeader
	as cpTimestamp					stamp
	as cpContactBufferHeader ptr	next
	as unsigned integer				numContacts
end type


type cpSpace
	'' *** User definable fields
	
	'' Number of iterations to use in the impulse solver to solve contacts.
	as integer	iterations
	
	'' Number of iterations to use in the impulse solver to solve elastic collisions.
	as integer	elasticIterations
	
	'' Default gravity to supply when integrating rigid body motions.
	as cpVect	gravity
	
	'' Default damping to supply when integrating rigid body motions.
	as cpFloat	damping
	
	'' Speed threshold for a body to be considered idle.
	'' The default value of 0 means to let the space guess a good threshold based on gravity.
	as cpFloat	idleSpeedThreshold
	
	'' Time a group of bodies must remain idle in order to fall asleep
	'' The default value of INFINITY disables the sleeping algorithm.
	as cpFloat	sleepTimeThreshold
	
	'' *** Internally Used Fields
	
	'' When the space is locked, you should not add or remove objects;
	as cpBool	locked
	
	'' Time stamp. Is incremented on every call to cpSpaceStep().
	as cpTimestamp	stamp

	'' The static and active shape spatial hashes.
	as cpSpaceHash ptr staticShapes, activeShapes

	
	'' List of bodies in the system.
	as cpArray ptr bodies
	
	'' List of groups of sleeping bodies.
	as cpArray ptr	sleepingComponents
	
	'' List of active arbiters for the impulse solver.
	as cpArray ptr arbiters, pooledArbiters
	
	'' Linked list ring of contact buffers.
	'' Head is the current buffer. Tail is the oldest buffer.
	'' The list points in the direction of tail->head.
	as cpContactBufferHeader ptr contactBuffersHead, contactBuffersTail
	
	'' List of buffers to be free()ed when destroying the space.
	as cpArray ptr allocatedBuffers
	
	'' Persistant contact set.
	as cpHashSet ptr contactSet
	
	'' List of constraints in the system.
	as cpArray ptr constraints
	
	'' Set of collisionpair functions.
	as cpHashSet ptr collFuncSet
	'' Default collision handler.
	as cpCollisionHandler defaultHandler
	
	as cpHashSet ptr postStepCallbacks
	
	as cpBody staticBody
end type

' Basic allocation/destruction functions.
declare function cpSpaceAlloc cdecl alias "cpSpaceAlloc" ( ) as cpSpace ptr
declare function cpSpaceInit cdecl alias "cpSpaceInit" ( byval spacep as cpSpace ptr ) as cpSpace ptr
declare function cpSpaceNew cdecl alias "cpSpaceNew" ( ) as cpSpace ptr

declare sub cpSpaceDestroy cdecl alias "cpSpaceDestroy" ( byval spacep as cpSpace ptr )
declare sub cpSpaceFree cdecl alias "cpSpaceFree" ( byval spacep as cpSpace ptr )

' Convenience function. Frees all referenced entities. (bodies, shapes and joints)
declare sub cpSpaceFreeChildren cdecl alias "cpSpaceFreeChildren" ( byval spacep as cpSpace ptr )

'' Needed to be defined after the cpSpace type was closed
private function cpBodyIsStatic( byval body as const cpBody ptr ) as cpBool
	dim s as cpSpace ptr
	s = body->space
	return ( s <> cptr( cpSpace ptr, 0 ) ) and ( body = @(s->staticBody) )
end function

'' Collision handler management functions.
declare sub cpSpaceSetDefaultCollisionHandler cdecl alias "cpSpaceSetDefaultCollisionHandler" ( _
	byval as cpSpace ptr, byval as cpCollisionBeginFunc, byval as cpCollisionPreSolveFunc, byval as cpCollisionPostSolveFunc, byval as cpCollisionSeparateFunc, byval as any ptr )
declare sub cpSpaceAddCollisionHandler cdecl alias "cpSpaceAddCollisionHandler" ( _
	byval as cpSpace ptr, byval as cpCollisionType, byval as cpCollisionType, _
	byval as cpCollisionBeginFunc, byval as cpCollisionPreSolveFunc, byval as cpCollisionPostSolveFunc, byval as cpCollisionSeparateFunc, byval as any ptr )
declare sub cpSpaceRemoveCollisionHandler cdecl alias "cpSpaceRemoveCollisionHandler" ( _
	byval as cpSpace ptr, byval as cpCollisionType, byval as cpCollisionType )

' Add and remove entities from the system.
declare function cpSpaceAddShape cdecl alias "cpSpaceAddShape" ( byval as cpSpace ptr, byval as cpShape ptr ) as cpShape ptr
declare function cpSpaceAddStaticShape cdecl alias "cpSpaceAddStaticShape" ( byval as cpSpace ptr, byval as cpShape ptr ) as cpShape ptr
declare function cpSpaceAddBody cdecl alias "cpSpaceAddBody" ( byval as cpSpace ptr, byval as cpBody ptr ) as cpBody ptr
declare function cpSpaceAddConstraint cdecl alias "cpSpaceAddConstraint" ( byval as cpSpace ptr, byval as cpConstraint ptr ) as cpConstraint ptr

declare sub cpSpaceRemoveShape cdecl alias "cpSpaceRemoveShape" ( byval as cpSpace ptr, byval as cpShape ptr )
declare sub cpSpaceRemoveStaticShape cdecl alias "cpSpaceRemoveStaticShape" ( byval as cpSpace ptr, byval as cpShape ptr )
declare sub cpSpaceRemoveBody cdecl alias "cpSpaceRemoveBody" ( byval as cpSpace ptr, byval as cpBody ptr )
declare sub cpSpaceRemoveConstraint cdecl alias "cpSpaceRemoveConstraint" ( byval as cpSpace ptr, byval as cpConstraint ptr )

'' Post Step function definition
type cpPostStepFunc as sub cdecl( byval as cpSpace ptr, byval as any ptr, byval as any ptr )
'' Register a post step function to be called after cpSpaceStep() has finished.
'' obj is used a key, you can only register one callback per unique value for obj
declare sub cpSpaceAddPostStepCallback( byval as cpSpace ptr, byval as cpPostStepFunc, byval as any ptr, byval as any ptr )

'' Point query callback function
type cpSpacePointQueryFunc as sub cdecl( byval as cpShape ptr, byval as any ptr )
declare sub cpSpacePointQuery cdecl alias "cpSpacePointQuery" ( byval as cpSpace ptr, byval as cpVect, byval as cpLayers, byval as cpGroup, byval as cpSpacePointQueryFunc, byval as any ptr )
declare function cpSpacePointQueryFirst cdecl alias "cpSpacePointQueryFirst" ( byval as cpSpace ptr, byval as cpVect, byval as cpLayers, byval as cpGroup ) as cpShape ptr

'' Segment query callback function
type cpSpaceSegmentQueryFunc as sub cdecl( byval as cpShape ptr, byval as cpFloat, byval as cpVect, byval as any ptr )
declare function cpSpaceSegmentQuery cdecl alias "cpSpaceSegmentQuery" ( byval as cpSpace ptr, byval as cpVect, byval as cpVect, byval as cpLayers, byval as cpGroup, byval as cpSpaceSegmentQueryFunc, byval as any ptr ) as cpBool
declare function cpSpaceSegmentQueryFirst cdecl alias "cpSpaceSegmentQueryFirst" ( byval as cpSpace ptr, byval as cpVect, byval as cpVect, byval as cpLayers, byval as cpGroup, byval as cpSegmentQueryInfo ptr ) as cpShape ptr

'' BB query callback function
type cpSpaceBBQueryFunc as sub cdecl( byval as cpSpace ptr, byval as any ptr )
declare sub cpSpaceBBQuery cdecl alias "cpSpaceBBQuery" ( byval as cpSpace ptr, byval as cpBB, byval as cpLayers, byval as cpGroup, byval as cpSpaceBBQueryFunc, byval as any ptr )

'' Iterator function for iterating the bodies in a space.
type cpSpaceBodyIterator as sub cdecl( byval as cpBody ptr, byval as any ptr )
declare sub cpSpaceEachBody cdecl alias "cpSpaceEachBody" ( byval as cpSpace ptr, byval as cpSpaceBodyIterator, byval as any ptr )

' Spatial hash management functions.
declare sub cpSpaceResizeStaticHash cdecl alias "cpSpaceResizeStaticHash" ( byval as cpSpace ptr, byval as cpFloat, byval as integer )
declare sub cpSpaceResizeActiveHash cdecl alias "cpSpaceResizeActiveHash" ( byval as cpSpace ptr, byval as cpFloat, byval as integer )
declare sub cpSpaceRehashStatic cdecl alias "cpSpaceRehashStatic" ( byval as cpSpace ptr )

' Update the space.
declare sub cpSpaceStep cdecl alias "cpSpaceStep" ( byval as cpSpace ptr, byval as cpFloat )

